@@ -16,17 +16,21 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import
+from __future__ import division
 
 import os
 import re
+import subprocess
 import sys
 import time
+import traceback
 
 import gobject
 import gtk
 import pango
 
 from solfege import cfg
+from solfege import osutils
 from solfege import soundcard
 
 import solfege
@@ -160,7 +164,7 @@ class nCheckButton(gtk.CheckButton, cfg.ConfigUtils):
         self.set_bool(self.m_name, self.get_active())
         if self.m_callback:
             self.m_callback(_o)
- 
+
 def RadioButton(group, label, callback=None):
     rdb = gtk.RadioButton(group, label)
     if callback:
@@ -494,6 +498,16 @@ def dialog_ok(text, parent=None, secondary_text=None, msgtype=gtk.MESSAGE_INFO):
     m.destroy()
     return ret
 
+def dialog_delete(text, parent, secondary_text=None):
+    m = gtk.MessageDialog(parent, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, gtk.BUTTONS_NONE, text)
+    m.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_DELETE, gtk.RESPONSE_ACCEPT)
+    if secondary_text:
+        m.format_secondary_text(secondary_text)
+    ret = m.run()
+    m.destroy()
+    return ret == gtk.RESPONSE_ACCEPT
+    return ret
+
 class NewLineBox(gtk.VBox):
     def __init__(self):
         gtk.VBox.__init__(self)
@@ -710,7 +724,7 @@ class EditorDialogBase(object):
                 # to do handle string to int conversion in
                 # PractiseSheet.parse_file. Just let int() raise ValueException
                 # if the string in not an integer.
-                solfege.win.display_exception_message(e)
+                display_exception_message(e)
         dialog.destroy()
     def get_save_as_dialog(self):
         dialog = gtk.FileChooserDialog(_("Save As..."), self,
@@ -806,34 +820,14 @@ class EditorDialogBase(object):
         return ret
 
 
-class LogWindow(gtk.Window):
-    def __init__(self, parent):
-        gtk.Window.__init__(self)
-        self.set_transient_for(parent)
-        self.set_destroy_with_parent(True)
-        self.set_modal(True)
-        vbox = gtk.VBox()
-        vbox.set_spacing(4)
-        self.add(vbox)
-        self.set_default_size(630, 400)
+class LogWidget(gtk.ScrolledWindow):
+    def __init__(self):
+        gtk.ScrolledWindow.__init__(self)
         self.g_textbuffer = gtk.TextBuffer()
         self.g_textbuffer.create_tag('h1', weight=pango.WEIGHT_BOLD,
             size=16*pango.SCALE)
         self.g_textview = gtk.TextView(buffer=self.g_textbuffer)
-        self.g_scrolledwindow = gtk.ScrolledWindow()
-        vbox.pack_start(self.g_scrolledwindow)
-        self.g_scrolledwindow.add(self.g_textview)
-        vbox.pack_start(gtk.HSeparator(), False, False)
-        bbox = gtk.HButtonBox()
-        bbox.set_layout(gtk.BUTTONBOX_END)
-        vbox.pack_start(bbox, False, False)
-        self.g_close_button = gtk.Button(stock="gtk-close")
-        self.g_close_button.set_sensitive(False)
-        self.g_close_button.connect('clicked', lambda w: self.destroy())
-        bbox.pack_start(self.g_close_button)
-        self.show_all()
-    def run_finished(self):
-        self.g_close_button.set_sensitive(True)
+        self.add(self.g_textview)
     def write(self, s, tag=None):
         if tag:
             self.g_textbuffer.insert_with_tags_by_name(
@@ -846,6 +840,55 @@ class LogWindow(gtk.Window):
         # This is needed to make the window update:
         while gtk.events_pending():
             gtk.main_iteration()
+    def popen(self, *args, **kwargs):
+        """
+        Return the Popen objects .returncode
+        """
+        self.write("$ %s\n" % " ".join(args[0]))
+        p = osutils.Popen(
+            stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT,
+            *args, **kwargs)
+        while 1:
+            p.poll()
+            # returncode != None means that the process has finished
+            if p.returncode != None:
+                break
+            while 1:
+                s = p.stdout.readline()
+                if not s:
+                    break
+                self.write(s)
+            time.sleep(1)
+        p.wait()
+        return p.returncode
+
+
+class LogWindow(gtk.Window):
+    def __init__(self, parent):
+        gtk.Window.__init__(self)
+        self.set_transient_for(parent)
+        self.set_destroy_with_parent(True)
+        self.set_modal(True)
+        vbox = gtk.VBox()
+        vbox.set_spacing(4)
+        self.add(vbox)
+        self.set_default_size(630, 400)
+        self.g_logwidget = LogWidget()
+        vbox.pack_start(self.g_logwidget)
+        vbox.pack_start(gtk.HSeparator(), False, False)
+        bbox = gtk.HButtonBox()
+        bbox.set_layout(gtk.BUTTONBOX_END)
+        vbox.pack_start(bbox, False, False)
+        self.g_close_button = gtk.Button(stock="gtk-close")
+        self.g_close_button.set_sensitive(False)
+        self.g_close_button.connect('clicked', lambda w: self.destroy())
+        bbox.pack_start(self.g_close_button)
+        self.show_all()
+        self.write = self.g_logwidget.write
+        self.popen = self.g_logwidget.popen
+    def run_finished(self):
+        self.g_close_button.set_sensitive(True)
 
 
 class ClickableLabel(gtk.LinkButton):
@@ -873,3 +916,131 @@ class ClickableLabel(gtk.LinkButton):
         im.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)
         self.set_image(im)
         self.set_alignment(0.0, 0.5)
+
+
+class ExceptionDialog(gtk.Dialog):
+    def __init__(self, exception):
+        gtk.Dialog.__init__(self, sys.exc_info()[0].__name__)
+        self.set_resizable(True)
+        self.set_border_width(6)
+        self.set_has_separator(False)
+        self.vbox.set_spacing(0)
+        hbox = gtk.HBox()
+        hbox.set_spacing(12)
+        hbox.set_border_width(6)
+        self.vbox.pack_start(hbox, False)
+        vbox = gtk.VBox()
+        hbox.pack_start(vbox, False, False)
+        img = gtk.image_new_from_stock(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_DIALOG)
+        vbox.pack_start(img, False, False)
+        vbox = gtk.VBox()
+        hbox.pack_start(vbox)
+        self.msg_vbox = gtk.VBox()
+        vbox.pack_start(self.msg_vbox)
+        self.g_primary = gtk.Label()
+        if sys.exc_info()[0].__name__ == 'AttributeError':
+            self.g_primary.set_name("DEBUGWARNING")
+        self.g_primary.set_alignment(0.0, 0.5)
+        self.g_primary.set_line_wrap(True)
+        self.m_primary_bold = False
+        self.msg_vbox.pack_start(self.g_primary)
+        if isinstance(exception, StandardError):
+            self.g_primary.set_text(str(exception).decode(sys.getfilesystemencoding(), 'replace'))
+        else:
+            if 'args' in dir(exception) and exception.args:
+                estr = exception.args[0]
+            else:
+                estr = exception
+            if not isinstance(estr, unicode):
+                estr = str(estr).decode(sys.getfilesystemencoding(), 'replace')
+            self.g_primary.set_text(estr)
+            if 'args' in dir(exception):
+                for s in exception.args[1:]:
+                    self.add_text(s)
+        # This label is here just for spacing
+        l = gtk.Label("")
+        vbox.pack_start(l)
+        expander = gtk.Expander("Traceback")
+        self.vbox.pack_start(expander)
+        l = gtk.Label("".join(traceback.format_exception(
+            sys.exc_type, sys.exc_value, sys.exc_traceback)).decode(sys.getfilesystemencoding(), 'replace'))
+        l.set_alignment(0.0, 0.5)
+        sc = gtk.ScrolledWindow()
+        sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        sc.set_size_request(-1, 100)
+        expander.add(sc)
+        sc.add_with_viewport(l)
+        l = gtk.Label(_("Visit http://www.solfege.org/support if you need help."))
+        l.set_alignment(0.0, 0.5)
+        vbox.pack_start(l, False)
+        w = self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+        self.set_default_response(gtk.RESPONSE_CLOSE)
+        self.set_focus(w)
+        self.show_all()
+    def _make_primary_bold(self):
+        if not self.m_primary_bold:
+            self.m_primary_bold = True
+            self.g_primary.set_markup('<span weight="bold" size="larger">%s</span>' %
+                escape(self.g_primary.get_text()))
+    def _parsep(self):
+        l = gtk.Label("")
+        l.show()
+        self.msg_vbox.pack_start(l)
+    def add_text(self, text):
+        self._make_primary_bold()
+        self._parsep()
+        # We add a empty string with a newline to get the spacing
+        l = gtk.Label(text)
+        l.set_line_wrap(True)
+        l.set_alignment(0.0, 0.5)
+        l.show()
+        self.msg_vbox.pack_start(l)
+    def add_nonwrapped_text(self, text):
+        self._make_primary_bold()
+        self._parsep()
+        sc = gtk.ScrolledWindow()
+        l = gtk.Label()
+        l.set_markup('<span font_family="monospace">%s</span>' % escape(text))
+        l.set_line_wrap(False)
+        l.set_alignment(0.0, 0.5)
+        l.show()
+        sc.add_with_viewport(l)
+        self.msg_vbox.pack_start(sc)
+        sc.show_all()
+        sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
+        w, h = l.get_ancestor(gtk.Viewport).size_request()
+        max_lines = 5
+        lines = text.count("\n") + 1
+        # This is the space the horizontal scrollbar takes
+        scrollbar_height = (sc.get_hscrollbar().size_request()[1]
+            + sc.style_get_property("scrollbar-spacing"))
+        if lines <= max_lines:
+            # make the scrollwin so high that it can display all lines
+            sc.set_size_request(-1, h)
+        else:
+            sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+            sc.set_size_request(-1, int(h * max_lines / lines))
+        adj = sc.get_hscrollbar().get_adjustment()
+        def f(scrollbar, event):
+            sc.set_size_request(-1, int(h * max_lines / lines) + scrollbar_height)
+            scrollbar.disconnect(self._hscroll_expose_id)
+        # f is called if the horizontal scrollbar is visible. This because we
+        # need to allocate space for the scrollbar too.
+        self._hscroll_expose_id = sc.get_hscrollbar().connect('expose-event', f)
+
+def display_exception_message(exception, lessonfile=None):
+        """Call this function only inside an except clause."""
+        sourcefile, lineno, func, code = traceback.extract_tb(sys.exc_info()[2])[0]
+        # We can replace characters because we will only display the
+        # file name, not open the file.
+        sourcefile = sourcefile.decode(sys.getfilesystemencoding(), 'replace')
+        m = ExceptionDialog(exception)
+        if lessonfile:
+            m.add_text(_("Please check the lesson file %s.") % lessonfile)
+        if sourcefile:
+            m.add_text(_('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno})
+        if 'm_nonwrapped_text' in dir(exception):
+            m.add_nonwrapped_text(exception.m_nonwrapped_text)
+        m.run()
+        m.destroy()
+
