@@ -51,7 +51,7 @@ _test_mode = False
 
 # The name of the folder containing the standard learning trees.
 # Relative to the installation dir.
-exercises_dir = u"exercises/standard"
+exercises_dir = os.path.join(u"exercises", u"standard")
 _abs_exercises_dir = os.path.join(os.getcwdu(), exercises_dir) + os.sep
 
 solfege_uri = u"solfege:"
@@ -505,8 +505,8 @@ class MpdParsable(MusicBaseClass):
                 if not score.voice11.is_bar_full():
                     logging.warning(u"bar check in question failed.")
         return score
-            
- 
+
+
 
 class MpdDisplayable(MpdParsable):
     pass
@@ -521,12 +521,12 @@ class MpdTransposable(MpdDisplayable):
         # We will display the first 3 non-empty lines
         v = [x.strip() for x in self.m_musicdata.split("\n") if x]
         v = v[:3]
-        raise MusicObjectException("%s\n%s" % 
+        raise MusicObjectException("%s\n%s" %
             ("get_first_pitch() could not find a musical pitch in the music data:",
              "\n".join(v)))
     def get_musicdata_transposed(self, lessonfile_ref):
         """
-        Return m_musicdata, but transposed relative to 
+        Return m_musicdata, but transposed relative to
         lesonfile_ref.m_transpose. All pitches identified as Lexer.NOTE
         are transposed.
         """
@@ -744,7 +744,7 @@ class Rvoice(VoiceCommon):
             # no translation
             return "\\staff\\relative %s{\n%s\n}" \
               % (errpitch, lex.m_string)
-            
+
         pitch.m_octave_i = 0
         lex.set_first_pitch(pitch)
         if lessonfile_ref.header.random_transpose[0]:
@@ -852,7 +852,7 @@ class Satb(ChordCommon):
 
 class PercBaseClass(MpdParsable):
     def get_mpd_music_string(self, lessonfile_ref):
-        return "\\staff{\n%s\n}" % self.m_musicdata
+        return "\\rhythmstaff{\n%s\n}" % self.m_musicdata
     def _gen_track(self, lessonfile_ref, question):
         try:
             score = mpd.parser.parse_to_score_object(self.get_mpd_music_string(lessonfile_ref))
@@ -1350,7 +1350,7 @@ class QuestionsLessonfile(LessonfileCommon):
         if we select "Override default instrument" should call this method.
         """
         assert count in (1, 2, 3)
-        if (count in (2, 3) and 
+        if (count in (2, 3) and
             cfg.get_bool('config/override_default_instrument')):
                 retval = [cfg.get_int('config/lowest_instrument'),
                           cfg.get_int('config/lowest_instrument_volume'),
@@ -1485,6 +1485,67 @@ class DictationLessonfile(QuestionsLessonfile):
 class SingChordLessonfile(QuestionsLessonfile):
     pass
 
+
+class RhythmDictation2Lessonfile(LessonfileCommon):
+    def parse_string(self, s, really_filename=None):
+        LessonfileCommon.parse_string(self, s, really_filename)
+        if not self.m_questions:
+            raise NoQuestionsInFileException(self.m_filename)
+        # Make sure the variables are lists. If there is only one elemt
+        # in the lesson file then we must make a list of them.
+        for question in self.m_questions:
+            if not isinstance(question['bars'], list):
+                question['bars'] = [question['bars']]
+            if not isinstance(question['elements'], list):
+                question['elements'] = [question['elements']]
+    def generate_random_question(self):
+        def rat_len_of_digits(digits):
+            """
+            Return a Rat representing the length of the digits.
+            "4 8 8" returns Rat(2, 4)
+            """
+            ret = Rat(0, 1)
+            for d in digits.split():
+                ret += Duration.new_from_string(d).get_rat_value()
+            return ret
+        self._idx = random.randint(0, len(self.m_questions) - 1)
+        # The score where the user enters his answer
+        self.m_answer_score = score = elems.Score()
+        score.add_staff(staff_class=elems.RhythmStaff)
+        for num, den in self.m_questions[self._idx]['bars']:
+            score.add_bar(elems.TimeSignature(num, den))
+        for bar in score.m_bars:
+            bar.fill_skips(score.voice11)
+        # the question being played
+        self.m_question_score = score = elems.Score()
+        score.add_staff(staff_class=elems.RhythmStaff)
+        for num, den in self.m_questions[self._idx]['bars']:
+            score.add_bar(elems.TimeSignature(num, den))
+            bar = self.m_question_score.m_bars[-1]
+            total = Rat(0, 1)
+            # We try only 1000 times, since it is possible that the lesson
+            # file is badly written with too long elements or too short bars.
+            count = 0
+            max_count = 100
+            while total < bar.m_timesig.as_rat() and count < max_count:
+                elem = random.choice(self.m_questions[self._idx]['elements'])
+                if total + rat_len_of_digits(elem) > bar.m_timesig.as_rat():
+                    count += 1
+                    continue
+                total += rat_len_of_digits(elem)
+                for e in elem.split(" "):
+                    e = e.strip()
+                    n = elems.Note.new_from_string(u"c'%s" % e)
+                    score.voice11.append(n)
+                    count = 0
+                count += 1
+            if count == max_count:
+                raise LessonfileException("Bad elements variable in the lessonfile. Too long elements.")
+    def play_question(self):
+        tracks = mpd.score_to_percussion_tracks(self.m_question_score)
+        tracks[0].prepend_bpm(*self.m_questions[self._idx]['tempo'])
+        soundcard.synth.play_track(*tracks)
+
 class NameIntervalLessonfile(HeaderLessonfile):
     def parse_string(self, s, really_filename=None):
         super(NameIntervalLessonfile, self).parse_string(s, really_filename)
@@ -1664,6 +1725,12 @@ class ElembuilderLessonfile(QuestionsLessonfile):
         super(ElembuilderLessonfile, self).parse_string(s, really_filename)
         # We need the name variable for statistics
         self.discard_questions_without_name()
+        # question['elements'] should always be a list. If a question allows
+        # a single element as answer one gets a dict from parse_string and not
+        # a list. Thus we pack the dict in a list and are happy.
+        for question in self.m_questions:
+            if type(question['elements']) <> (type([])):
+                question['elements'] = [question['elements']]
         # This loop let us use normal strings and lesson file functions
         # like pangomarkup as element labels.
         for question in self.m_questions:
@@ -1699,7 +1766,7 @@ class InfoCache(object):
                 self.parse_file(filename)
             return self._data[filename][field]
         def parse_file(self, filename):
-            from solfege import frontpage 
+            from solfege import frontpage
             try:
                 p = frontpage.load_tree(filename)
                 self._data[filename] = {'title': p.m_name,
@@ -1725,19 +1792,22 @@ class InfoCache(object):
             self.parse_file(filename, mtime)
         return self._data[filename][field]
     def parse_file(self, filename, mtime=None):
-        logging.debug("parse_file %s" % filename)
         assert is_uri(filename) or os.path.isabs(filename)
         p = parse_lesson_file_header(uri_expand(filename))
         if not p:
             raise self.FileNotLessonfile(filename)
-        self._data[filename] = {
-            'title': p.header.get('title', 'error: no title in file'),
-            'module': p.header['module'],
-            'test': p.header.get('test', None),
-            'test_requirement': p.header.get('test_requirement', None),
-            'mtime': mtime if mtime else os.path.getmtime(uri_expand(filename)),
-            'replaces': p.header.get('replaces', []),
+        try:
+            self._data[filename] = {
+                'title': p.header.get('title', 'error: no title in file'),
+                'module': p.header['module'],
+                'test': p.header.get('test', None),
+                'test_requirement': p.header.get('test_requirement', None),
+                'mtime': mtime if mtime else os.path.getmtime(uri_expand(filename)),
+                'replaces': p.header.get('replaces', []),
         }
+        except KeyError:
+            logging.debug("InfoCache.parse_file: FileNotLessonfile(%s)" % filename)
+            raise self.FileNotLessonfile(filename)
     def iter_parse_all_files(self):
         """
         Parse the files and put then in the cache.
