@@ -17,9 +17,11 @@
 from __future__ import absolute_import
 import gtk
 import gobject
-from solfege.mpd import parser
-from solfege.mpd import engravers
+import pango
 from solfege.mpd import const
+from solfege.mpd import elems
+from solfege.mpd import engravers
+from solfege.mpd import parser
 from solfege.mpd.rat import Rat
 
 class MusicDisplayer(gtk.ScrolledWindow):
@@ -27,7 +29,7 @@ class MusicDisplayer(gtk.ScrolledWindow):
         gtk.ScrolledWindow.__init__(self)
         self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
         self.m_callback = None
-        self.m_engravers = [[],]
+        self.m_engravers = []
         self.m_fontsize = 20
         self.m_clickables = []
         self.g_d = gtk.DrawingArea()
@@ -38,49 +40,44 @@ class MusicDisplayer(gtk.ScrolledWindow):
         self.g_d.add_events(gtk.gdk.EXPOSURE_MASK | gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.BUTTON_PRESS_MASK|gtk.gdk.POINTER_MOTION_MASK)
         self.g_d.connect("button_press_event", self.on_button_press_event)
 
-        self.g_pixmap = None
         self.m_width = self.m_height = 0
     def clear(self, numstaff=1):
         if numstaff == 0:
             self.m_engravers = []
         else:
-            self.m_engravers = [[],]*numstaff
+            sc = elems.Score()
+            for x in range(numstaff):
+                sc.add_staff()
+            self.m_engravers = engravers.ScoreContext(sc).m_contexts
         self.g_d.queue_draw()
-        self.g_pixmap = None
         dim = engravers.dimentions[20]
         self.set_size_request(self.get_size_request()[0], numstaff*dim.staff_spacing+dim.first_staff_ypos)
-    def display_range(self, music, fontsize, start, end):
-        """Exception handling should be done by the caller."""
-        #FIXME this function only handles music where start == Rat(0, 1) correctly
-        assert start == Rat(0, 1)
-        score = parser.parse_to_score_object(music)
-        tv = score.m_timeposdict.keys()
-        v = [t for t in tv if t < end]
-        v.sort()
-        self.m_fontsize = fontsize
-        self.m_engravers = score._generate_engravers(v, fontsize)
-        self._display()
-    def display(self, music, fontsize, what_part=const.ALL):
+    def display(self, music, fontsize, last_timepos=None):
         """Exception handling should be done by the caller."""
         score = parser.parse_to_score_object(music)
-        sc = engravers.ScoreContext(score, what_part)
+        sc = engravers.ScoreContext(score, last_timepos)
         self.m_engravers = sc.m_contexts
         self.m_fontsize = fontsize
         self._display()
     def display_score(self, score):
-        self.m_engravers = engravers.ScoreContext(score, const.ALL).m_contexts
+        self.m_engravers = engravers.ScoreContext(score).m_contexts
         self._display()
     def _display(self):
         dim = engravers.dimentions[self.m_fontsize]
-        if self.m_engravers and self.m_engravers[0]:
-            self.m_width = max([e.m_xpos for e in self.m_engravers[0] if not isinstance(e, (engravers.BeamEngraver, engravers.TupletEngraver, engravers.TieEngraver))]) + 20
+        if self.m_engravers:
+            self.m_width = 0
+            for eng in self.m_engravers:
+                if eng:
+                    self.m_width = max(self.m_width, max([e.m_xpos for e in eng if not isinstance(e, (engravers.BeamEngraver, engravers.TupletEngraver, engravers.TieEngraver))]))
+                    if not isinstance(eng[-1], engravers.BarlineEngraver):
+                        self.m_width += 20
             self.m_height = len(self.m_engravers)*dim.staff_spacing+dim.first_staff_ypos
         else:
+            # We get here if we try to display a score with no staffs.
             self.m_width = 100
-            self.m_height = dim.staff_spacing * 2
+            self.m_height = dim.staff_spacing + dim.first_staff_ypos
         self.set_size_request(self.get_size_request()[0], self.m_height)
         self.g_d.set_size_request(self.m_width, self.m_height-4)
-        self.g_pixmap = None
         self.g_d.queue_draw()
     def add_clickable_region(self, x, y, w, h, midi_int):
         self.m_clickables.append({'x':x, 'y':y, 'w':w, 'h':h,
@@ -94,53 +91,45 @@ class MusicDisplayer(gtk.ScrolledWindow):
     def on_expose_event(self, darea, event):
         self.white_gc = self.g_d.get_style().white_gc
         self.black_gc = self.g_d.get_style().black_gc
-        staff_len = 100
-        for staff_engravers in self.m_engravers:
-            for e in reversed(staff_engravers):
-                # Usually there is a barline engraver at the end of the
-                # list, but if the staff ends without a barline, then
-                # there is no. So we add some extra space, so that the
-                # last note is not at the very end of the staff line.
-                if isinstance(e, engravers.BarlineEngraver):
-                    staff_len = max(staff_len, e.m_xpos)
-                else:
-                    staff_len = max(staff_len, e.m_xpos + 20)
-                break
+        staff_len = self.m_width
+        if staff_len == 0:
+            staff_len = darea.window.get_size()[0]
         dim = engravers.dimentions[self.m_fontsize]
-        #if self.g_pixmap is None:
-        if 1==1:
-            d = self.get_allocation()
-            if self.m_width < d[2]:
-                self.m_width = d[2]
-            if self.m_height < d[3]:
-                self.m_height = d[3]
-            #self.g_pixmap = pixmap = gtk.create_pixmap(self.g_d,
-            #                         self.m_width, self.m_height)
-            darea.window.draw_rectangle(self.white_gc, True, 0, 0,
-                               self.m_width, self.m_height)
-            staff_centrum = dim.first_staff_ypos
-            self.m_clickables = []
-            for staff in self.m_engravers:
-                for e in staff:
-                    if isinstance(e, engravers.StemEngraver):
-                        continue
+        d = self.get_allocation()
+        if self.m_width < d[2]:
+            self.m_width = d[2]
+        if self.m_height < d[3]:
+            self.m_height = d[3]
+        darea.window.draw_rectangle(self.white_gc, True, 0, 0,
+                           self.m_width, self.m_height)
+        staff_centrum = dim.first_staff_ypos
+        self.m_clickables = []
+        for staff in self.m_engravers:
+            if staff.m_label:
+                ct = darea.create_pango_context()
+                l = pango.Layout(ct)
+                l.set_text(staff.m_label)
+                l.set_font_description(pango.FontDescription("Sans 10"))
+                darea.window.draw_layout(self.black_gc, 5, staff_centrum - engravers.dim20.linespacing * 6, l)
+            for e in staff:
+                if isinstance(e, engravers.StemEngraver):
+                    continue
+                e.engrave(darea, self.black_gc, staff_centrum)
+            # stems has to be drawn after noteheads and accidentals
+            for e in staff:
+                if isinstance(e, engravers.StemEngraver):
                     e.engrave(darea, self.black_gc, staff_centrum)
-                # stems has to be drawn after noteheads and accidentals
-                for e in staff:
-                    if isinstance(e, engravers.StemEngraver):
-                        e.engrave(darea, self.black_gc, staff_centrum)
-                # staff lines:
-                if isinstance(staff, engravers.RhythmStaffContext):
-                    linerange = (0,)
-                else:
-                    linerange = range(-2, 3)
-                for y in linerange:
-                    darea.window.draw_line(self.black_gc,
-                                 0, staff_centrum+dim.linespacing*y,
-                                 staff_len, staff_centrum+y*dim.linespacing)
-                staff_centrum = staff_centrum + dim.staff_spacing
-        return##############
-        darea.draw_pixmap(self.white_gc, self.g_pixmap, 0, 0, 0, 0, self.m_width, self.m_height)
+            # staff lines:
+            if isinstance(staff, engravers.RhythmStaffContext):
+                linerange = (0,)
+            else:
+                linerange = range(-2, 3)
+            for y in linerange:
+                darea.window.draw_line(self.black_gc,
+                             0, staff_centrum+dim.linespacing*y,
+                             staff_len, staff_centrum+y*dim.linespacing)
+            staff_centrum = staff_centrum + dim.staff_spacing
+
 
 class ChordEditor(MusicDisplayer):
     def __init__(self):
