@@ -108,18 +108,18 @@ def initialise_alsa_sequencer(port, verbose_init=0):
     synth = alsa_sequencer.AlsaSequencer(port, verbose_init)
 
 
-def initialise_external_midiplayer(cmdline, verbose_init=0):
+def initialise_external_midiplayer(verbose_init=0):
     global synth
     import solfege.soundcard.midifilesynth
     solfege.mpd.track.set_patch_delay = cfg.get_int("app/set_patch_delay")
-    synth = solfege.soundcard.midifilesynth.MidiFileSynth(cmdline, verbose_init)
+    synth = solfege.soundcard.midifilesynth.MidiFileSynth(verbose_init)
 
 
 def initialise_devicefile(devicefile, devicenum=0, verbose_init=0):
     global synth
     if devicefile == '/dev/sequencer2' or devicefile == '/dev/music':
         import solfege.soundcard.oss_sequencer2
-        synth = solfege.soundcard.oss_sequencer2.OSSSequencer2Synth(devicefile, devicenum, 
+        synth = solfege.soundcard.oss_sequencer2.OSSSequencer2Synth(devicefile, devicenum,
                                                   verbose_init)
     else:#if devicefile == '/dev/sequencer':
         if devicefile != '/dev/sequencer':
@@ -153,7 +153,7 @@ def end_export():
 instrument_sections = (
     'piano',
     'cromatic percussion',
-    'organ', 
+    'organ',
     'guitar',
     'bass',
     'strings',
