@@ -19,7 +19,7 @@ def f(s):
     else:
         print s
 
-sys.displayhook = f
+sys.__displayhook__ = f
 
 from solfege import testlib
 import solfege.i18n
@@ -27,6 +27,7 @@ solfege.i18n.setup(".")
 
 from solfege import lessonfile
 lessonfile.manager = lessonfile.LessonFileManager()
+lessonfile.infocache = lessonfile.InfoCache()
 import solfege.statistics
 solfege.db = solfege.statistics.DB(None)
 
@@ -51,8 +52,7 @@ os.mkdir(testlib.outdir)
 
 from solfege import soundcard
 from solfege.osutils import *
-soundcard.initialise_external_midiplayer(
-    cfg.get_string("sound/midi_player"))
+soundcard.initialise_external_midiplayer()
 
 soundcard.synth.start_testmode()
 
